@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portactivate.tcl
-# $Id: portactivate.tcl 109245 2013-08-11 11:51:15Z cal@macports.org $
+# $Id: portactivate.tcl 116820 2014-02-07 23:39:28Z cal@macports.org $
 #
 # Copyright (c) 2005, 2007, 2009-2011 The MacPorts Project
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
@@ -43,6 +43,7 @@ target_state ${org.macports.activate} no
 target_provides ${org.macports.activate} activate
 target_requires ${org.macports.activate} main archivefetch fetch checksum extract patch configure build destroot install
 target_prerun ${org.macports.activate} portactivate::activate_start
+target_postrun ${org.macports.activate} portactivate::activate_finish
 
 namespace eval portactivate {
 }
@@ -97,10 +98,18 @@ proc portactivate::activate_main {args} {
         ui_notice ""
     }
 
+    return 0
+}
+
+proc portactivate::activate_finish {args} {
+    global subport startupitem.autostart UI_PREFIX
+
+    # Do this _after_ activate_main, because post-activate hooks might create
+    # the files needed for this
     if {[tbool startupitem.autostart]} {
-        ui_notice "$UI_PREFIX [format [msgcat::mc "Loading %s"] [option subport]]"
+        ui_notice "$UI_PREFIX [format [msgcat::mc "Loading %s"] $subport]"
         if {[eval_targets "load"]} {
-            ui_error [format [msgcat::mc "Failed to load %s"] [option subport]]
+            ui_error [format [msgcat::mc "Failed to load %s"] $subport]
             return 1
         }
     }
