@@ -1,7 +1,7 @@
 # et:ts=4
 # portdistcheck.tcl
 #
-# $Id: portdistcheck.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
+# $Id: portdistcheck.tcl 116449 2014-01-25 16:57:17Z cal@macports.org $
 #
 # Copyright (c) 2007-2011 The MacPorts Project
 # Copyright (c) 2005-2006 Paul Guyot <pguyot@kallisys.net>,
@@ -47,27 +47,21 @@ namespace eval portdistcheck {
 }
 
 # define options
-options distcheck.type distcheck.check
+options distcheck.type
 
 # defaults
 default distcheck.type moddate
-default distcheck.check moddate
-
-# Deprecation
-option_deprecate distcheck.check distcheck.type
 
 proc portdistcheck::distcheck_main {args} {
-    global distcheck.type
-    global fetch.type
-    global fetch.ignore_sslcert
-    global subport portpath
+    global distcheck.type fetch.type fetch.ignore_sslcert \
+           subport portpath
 
     set port_moddate [file mtime ${portpath}/Portfile]
 
     ui_debug "Portfile modification date is [clock format $port_moddate]"
 
     set curl_options {}
-    if [tbool fetch.ignore_sslcert] {
+    if {[tbool fetch.ignore_sslcert]} {
         lappend curl_options "--ignore-ssl-cert"
     }
 
@@ -95,7 +89,7 @@ proc portdistcheck::distcheck_main {args} {
                         ui_warn "couldn't fetch $file_url for $subport ($error)"
                     } else {
                         if {$urlnewer} {
-                            ui_warn "port $subport: $file_url is newer than portfile"
+                            ui_warn "port $subport: $file_url is newer than Portfile"
                         }
                         incr count
                     }
@@ -133,13 +127,13 @@ proc portdistcheck::distcheck_main {args} {
                 set size $totalsize
                 set humansize "${size}"
             } elseif {$totalsize < 1024*1024} {
-                set size [expr $totalsize / 1024.0]
+                set size [expr {$totalsize / 1024.0}]
                 set humansize [format "%.1fK" $size]
             } elseif {$totalsize < 1024*1024*1024} {
-                set size [expr $totalsize / (1024.0*1024.0)]
+                set size [expr {$totalsize / (1024.0*1024.0)}]
                 set humansize [format "%.1fM" $size]
             } else {
-                set size [expr $totalsize / (1024.0*1024.0*1024.0)]
+                set size [expr {$totalsize / (1024.0*1024.0*1024.0)}]
                 set humansize [format "%.1fG" $size]
             }
             ui_msg "$subport: $humansize"
